'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getHeartbeat = getHeartbeat;

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _utils = require('../utils');

var utils = _interopRequireWildcard(_utils);

var _server = require('../server');

var server = _interopRequireWildcard(_server);

var _mediators = require('../model/mediators');

var _util = require('util');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

async function getHeartbeat(ctx) {
  try {
    const uptime = (0, _util.promisify)(server.getUptime);
    const result = await uptime();

    const mediators = await _mediators.MediatorModelAPI.find().exec();
    for (const mediator of Array.from(mediators)) {
      if (!result.mediators) {
        result.mediators = {};
      }

      if (mediator._lastHeartbeat != null && mediator._uptime != null &&
      // have we received a heartbeat within the last minute?
      (0, _moment2.default)().diff(mediator._lastHeartbeat, 'seconds') <= 60) {
        result.mediators[mediator.urn] = mediator._uptime;
      } else {
        result.mediators[mediator.urn] = null;
      }
    }

    result.now = Date.now();
    ctx.body = result;
  } catch (e) {
    return utils.logAndSetResponse(ctx, 500, `Error: ${e}`, 'error');
  }
}
//# sourceMappingURL=heartbeat.js.map