'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.config = undefined;

var _nconf = require('nconf');

var _nconf2 = _interopRequireDefault(_nconf);

var _appRootPath = require('app-root-path');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * Define the default constructor
 */
function Config() {
  // Get the argument-value to use
  _nconf2.default.argv().env('_');
  const environment = _nconf2.default.get('NODE:ENV');
  const conf = getConfigOverride(process.argv);

  // Load the configuration-values
  // user specified config override
  if (conf) {
    _nconf2.default.file('customConfigOverride', conf);
  }

  // environment override
  if (environment) {
    _nconf2.default.file('environmentOverride', `${_appRootPath.path}/config/${environment}.json`);
  }

  // load the default config file
  _nconf2.default.file('default', `${_appRootPath.path}/config/default.json`);

  // Return the result
}

function getConfigOverride(argv) {
  const [conf] = argv.filter(v => /^conf/.test(v));
  if (conf == null) {
    return;
  }

  if (/=(.*)/.test(conf)) {
    const [, value] = conf.match(/=(.*)/);
    return value;
  }

  const index = argv.indexOf(conf);
  if (argv.length > index + 1) {
    return argv[index + 1];
  }
}

/*
 * This function return the value that was set in the key-value store
 */
Config.prototype.get = key => _nconf2.default.get(key);

/*
 * This function constructs a new instanse of this class
 */
const config = exports.config = new Config();
//# sourceMappingURL=config.js.map